# -*- coding: utf-8 -*-
# Generated by Django 1.11.13 on 2018-10-04 10:41
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('marketplace', '0039_split_component_model'),
    ]

    operations = [
        migrations.CreateModel(
            name='CategoryColumn',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('index', models.PositiveSmallIntegerField(help_text='Index allows to reorder columns.')),
                ('title', models.CharField(help_text='Title is rendered as column header.', max_length=255)),
                ('attribute', models.CharField(blank=True, help_text='Resource attribute is rendered as table cell.', max_length=255)),
                ('widget', models.CharField(blank=True, help_text='Widget field allows to customise table cell rendering.', max_length=255)),
                ('category', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='columns', to='marketplace.Category')),
            ],
            options={
                'ordering': ('category', 'index'),
            },
        ),
    ]
