# -*- coding: utf-8 -*-
# Generated by Django 1.11.13 on 2018-09-17 18:15
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('marketplace', '0035_component_name_unit_description'),
    ]

    operations = [
        migrations.AddField(
            model_name='serviceprovider',
            name='api_secret_code',
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
        migrations.AlterField(
            model_name='serviceprovider',
            name='customer',
            field=models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to='structure.Customer'),
        ),
    ]
