# -*- coding: utf-8 -*-
# Generated by Django 1.11.13 on 2018-10-12 09:33
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import waldur_mastermind.invoices.models


class Migration(migrations.Migration):

    dependencies = [
        ('packages', '0014_unique_package_tenant'),
        ('invoices', '0024_immutable_default_json'),
    ]

    operations = [
        migrations.CreateModel(
            name='ServiceDowntime',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('start', models.DateTimeField(default=waldur_mastermind.invoices.models.get_default_downtime_start, help_text='Date and time when downtime has started.')),
                ('end', models.DateTimeField(default=django.utils.timezone.now, help_text='Date and time when downtime has ended.')),
                ('package', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='packages.OpenStackPackage')),
            ],
        ),
    ]
