# -*- coding: utf-8 -*-
# Generated by Django 1.11.21 on 2019-08-29 08:49
from __future__ import unicode_literals

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('marketplace', '0091_offering_paused_reason'),
    ]

    operations = [
        migrations.AlterField(
            model_name='attribute',
            name='key',
            field=models.CharField(max_length=255, primary_key=True, serialize=False, validators=[django.core.validators.RegexValidator('^[a-zA-Z][a-zA-Z0-9_]+$')]),
        ),
        migrations.AlterField(
            model_name='attributeoption',
            name='key',
            field=models.CharField(max_length=255, validators=[django.core.validators.RegexValidator('^[a-zA-Z][a-zA-Z0-9_]+$')]),
        ),
        migrations.AlterField(
            model_name='categorycomponent',
            name='type',
            field=models.CharField(help_text='Unique internal name of the measured unit, for example floating_ip.', max_length=50, validators=[django.core.validators.RegexValidator('^[a-zA-Z][a-zA-Z0-9_]+$')]),
        ),
        migrations.AlterField(
            model_name='offeringcomponent',
            name='type',
            field=models.CharField(help_text='Unique internal name of the measured unit, for example floating_ip.', max_length=50, validators=[django.core.validators.RegexValidator('^[a-zA-Z][a-zA-Z0-9_]+$')]),
        ),
    ]
