# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2017-06-20 16:16
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('structure', '0052_customer_subnets'),
        ('invoices', '0011_add_article_code'),
    ]

    operations = [
        migrations.AddField(
            model_name='offeringitem',
            name='project',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='structure.Project'),
        ),
        migrations.AddField(
            model_name='offeringitem',
            name='project_name',
            field=models.CharField(blank=True, max_length=150),
        ),
        migrations.AddField(
            model_name='offeringitem',
            name='project_uuid',
            field=models.CharField(blank=True, max_length=32),
        ),
        migrations.AddField(
            model_name='openstackitem',
            name='project',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='structure.Project'),
        ),
        migrations.AddField(
            model_name='openstackitem',
            name='project_name',
            field=models.CharField(blank=True, max_length=150),
        ),
        migrations.AddField(
            model_name='openstackitem',
            name='project_uuid',
            field=models.CharField(blank=True, max_length=32),
        ),
    ]
