class JupyterHubManagementConstants(object):
    SYNC_CONFIGURATION = 'install_jupyter'
    MAKE_VIRTUAL_ENV_GLOBAL = 'make_virtual_env_global'
    MAKE_VIRTUAL_ENV_LOCAL = 'make_virtual_env_local'
    DELETE_JUPYTER_HUB = 'delete_jupyter_hub'
