# -*- coding: utf-8 -*-
# Generated by Django 1.11.7 on 2018-07-03 17:09
from __future__ import unicode_literals

from django.db import migrations
import waldur_core.core.fields


class Migration(migrations.Migration):

    dependencies = [
        ('openstack_tenant', '0033_unique_instance_backend_id'),
    ]

    operations = [
        migrations.AlterField(
            model_name='instance',
            name='action_details',
            field=waldur_core.core.fields.JSONField(default=dict),
        ),
        migrations.AlterField(
            model_name='snapshot',
            name='action_details',
            field=waldur_core.core.fields.JSONField(default=dict),
        ),
        migrations.AlterField(
            model_name='subnet',
            name='allocation_pools',
            field=waldur_core.core.fields.JSONField(default=dict),
        ),
        migrations.AlterField(
            model_name='subnet',
            name='dns_nameservers',
            field=waldur_core.core.fields.JSONField(default=list, help_text='List of DNS name servers associated with the subnet.'),
        ),
        migrations.AlterField(
            model_name='volume',
            name='action_details',
            field=waldur_core.core.fields.JSONField(default=dict),
        ),
    ]
