# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2017-03-02 16:08
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('openstack_tenant', '0022_floatingip_internal_ip'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='instance',
            name='external_ip',
        ),
    ]
