# -*- coding: utf-8 -*-
# Generated by Django 1.11.7 on 2018-08-21 12:03
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('marketplace', '0021_remove_default_draft_state_from_order'),
    ]

    operations = [
        migrations.AddField(
            model_name='section',
            name='is_standalone',
            field=models.BooleanField(default=False, help_text='Whether section is rendered as a separate tab.'),
        ),
    ]
