<html>
<head lang="en">
    <meta charset="UTF-8">
    <title>The issue you have has been updated</title>
</head>
<body>
<p>
    Hello!
</p>
<p>
    Please visit <a href="{{ issue_url }}">{{ site_name }}</a> to find out more details.
</p>
</body>
</html>
