# -*- coding: utf-8 -*-
# Generated by Django 1.11.7 on 2018-06-26 07:11
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('marketplace', '0007_screenshots'),
    ]

    operations = [
        migrations.AddField(
            model_name='offering',
            name='native_description',
            field=models.CharField(blank=True, default='', max_length=500),
        ),
        migrations.AddField(
            model_name='offering',
            name='native_name',
            field=models.CharField(blank=True, default='', max_length=160),
        ),
        migrations.AddField(
            model_name='offering',
            name='preferred_language',
            field=models.CharField(blank=True, max_length=10),
        ),
    ]
