# -*- coding: utf-8 -*-
# Generated by Django 1.11.7 on 2018-07-06 16:54
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('marketplace', '0013_provider_to_customer'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='offering',
            name='preferred_language',
        ),
    ]
