# -*- coding: utf-8 -*-
# Generated by Django 1.11.20 on 2019-05-08 06:56
from __future__ import unicode_literals

from django.db import migrations, models

from ..utils import fill_activated_field


class Migration(migrations.Migration):

    dependencies = [
        ('marketplace', '0081_allow_ico_mime_type_for_images'),
    ]

    operations = [
        migrations.AddField(
            model_name='orderitem',
            name='activated',
            field=models.DateTimeField(blank=True, null=True, verbose_name='activation date'),
        ),
        migrations.AlterField(
            model_name='offeringcomponent',
            name='billing_type',
            field=models.CharField(choices=[('fixed', 'Fixed-price'), ('usage', 'Usage-based'), ('one', 'One-time'), ('few', 'One-time on plan switch')], default='fixed', max_length=5),
        ),
        migrations.RunPython(fill_activated_field, elidable=True),
    ]
