# -*- coding: utf-8 -*-
# Generated by Django 1.11.21 on 2019-08-21 09:38
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('marketplace', '0090_resource_current_usages'),
    ]

    operations = [
        migrations.AddField(
            model_name='offering',
            name='paused_reason',
            field=models.TextField(blank=True),
        ),
    ]
