# -*- coding: utf-8 -*-
# Generated by Django 1.11.13 on 2018-10-23 12:11
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('support', '0028_ignoredissuestatus_templatestatusnotification'),
    ]

    operations = [
        migrations.AddField(
            model_name='issue',
            name='resolution_date',
            field=models.DateTimeField(blank=True, null=True),
        ),
    ]
