# -*- coding: utf-8 -*-
# Generated by Django 1.11.16 on 2019-01-22 12:41
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import model_utils.fields


class Migration(migrations.Migration):

    dependencies = [
        ('marketplace', '0051_resource_backend_metadata'),
    ]

    operations = [
        migrations.CreateModel(
            name='ResourcePlanPeriod',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('start', models.DateTimeField(blank=True, null=True, verbose_name='start')),
                ('end', models.DateTimeField(blank=True, null=True, verbose_name='end')),
                ('plan', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='marketplace.Plan')),
                ('resource', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='marketplace.Resource')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]
