# -*- coding: utf-8 -*-
# Generated by Django 1.11.7 on 2018-07-20 15:32
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('marketplace', '0015_rename_order_item'),
    ]

    operations = [
        migrations.CreateModel(
            name='AttributeOption',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('key', models.CharField(max_length=255)),
                ('title', models.CharField(max_length=255)),
            ],
        ),
        migrations.RemoveField(
            model_name='attribute',
            name='available_values',
        ),
        migrations.AddField(
            model_name='attributeoption',
            name='attribute',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='options', to='marketplace.Attribute'),
        ),
        migrations.AlterUniqueTogether(
            name='attributeoption',
            unique_together=set([('attribute', 'key')]),
        ),
    ]
