# -*- coding: utf-8 -*-
# Generated by Django 1.11.24 on 2019-10-03 12:01
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('waldur_rancher', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='cluster',
            name='backend_id',
            field=models.CharField(blank=True, default='', max_length=255),
            preserve_default=False,
        ),
    ]
