# -*- coding: utf-8 -*-
# Generated by Django 1.11.20 on 2019-06-19 18:00
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('marketplace', '0088_fill_billing_period'),
    ]

    operations = [
        migrations.AlterField(
            model_name='componentusage',
            name='billing_period',
            field=models.DateField(),
        ),
    ]
