# -*- coding: utf-8 -*-
# Generated by Django 1.11.18 on 2019-02-27 10:00
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('marketplace', '0057_category_backend_id'),
    ]

    operations = [
        migrations.AddField(
            model_name='offering',
            name='terms_of_service',
            field=models.CharField(blank=True, default='', max_length=500),
        ),
    ]
