# -*- coding: utf-8 -*-
# Generated by Django 1.11.20 on 2019-04-12 15:42
from __future__ import unicode_literals

from decimal import Decimal
import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('marketplace', '0077_plan_period_uuid_unique'),
    ]

    operations = [
        migrations.AlterField(
            model_name='plancomponent',
            name='price',
            field=models.DecimalField(decimal_places=7, default=0, max_digits=15, validators=[django.core.validators.MinValueValidator(Decimal('0'))], verbose_name='Price per unit per billing period.'),
        ),
    ]
