# -*- coding: utf-8 -*-
# Generated by Django 1.11.18 on 2019-02-04 12:14
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import model_utils.fields
import waldur_core.core.fields
import waldur_core.core.validators


class Migration(migrations.Migration):

    dependencies = [
        ('marketplace', '0054_categorycomponentusage'),
    ]

    operations = [
        migrations.CreateModel(
            name='OfferingFile',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('name', models.CharField(max_length=150, validators=[waldur_core.core.validators.validate_name], verbose_name='name')),
                ('uuid', waldur_core.core.fields.UUIDField()),
                ('file', models.FileField(upload_to='offering_files')),
                ('offering', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='files', to='marketplace.Offering')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]
