# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('invoices', '0026_invoice__file'),
        ('contenttypes', '0002_remove_content_type_name'),
    ]

    operations = []
