# -*- coding: utf-8 -*-
# Generated by Django 1.11.18 on 2019-03-28 14:32
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('marketplace', '0072_offering_component_product_code'),
    ]

    operations = [
        migrations.AddField(
            model_name='componentusage',
            name='plan_period',
            field=models.ForeignKey(default=None, on_delete=django.db.models.deletion.CASCADE, to='marketplace.ResourcePlanPeriod', related_name='components', null=True),
            preserve_default=False,
        ),
        migrations.AlterUniqueTogether(
            name='componentusage',
            unique_together=set([('resource', 'component', 'plan_period')]),
        ),
    ]
