# -*- coding: utf-8 -*-
# Generated by Django 1.11.7 on 2018-07-03 17:02
from __future__ import unicode_literals

from django.db import migrations
import waldur_core.core.fields


class Migration(migrations.Migration):

    dependencies = [
        ('invoices', '0023_invoice_current_cost'),
    ]

    operations = [
        migrations.AlterField(
            model_name='genericinvoiceitem',
            name='details',
            field=waldur_core.core.fields.JSONField(blank=True, default=dict, help_text='Stores data about scope'),
        ),
        migrations.AlterField(
            model_name='offeringitem',
            name='offering_details',
            field=waldur_core.core.fields.JSONField(blank=True, default=dict, help_text='Stores data about offering'),
        ),
        migrations.AlterField(
            model_name='openstackitem',
            name='package_details',
            field=waldur_core.core.fields.JSONField(blank=True, default=dict, help_text='Stores data about package'),
        ),
    ]
