# -*- coding: utf-8 -*-
# Generated by Django 1.11.18 on 2019-04-02 08:53
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('openstack_tenant', '0040_volume_type'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='volume',
            name='type',
        ),
        migrations.AddField(
            model_name='volume',
            name='type',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL,
                                    to='openstack_tenant.VolumeType'),
        ),
    ]
