# -*- coding: utf-8 -*-
# Generated by Django 1.11.9 on 2018-03-22 14:51
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('python_management', '0002_generalized_instance_added_system_user'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='pythonmanagement',
            unique_together=set([('instance_content_type', 'instance_object_id', 'virtual_envs_dir_path', 'system_user')]),
        ),
    ]
