# -*- coding: utf-8 -*-
# Generated by Django 1.11.7 on 2018-07-30 15:37
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('openstack', '0039_immutable_default_json'),
    ]

    operations = [
        migrations.AlterField(
            model_name='floatingip',
            name='address',
            field=models.GenericIPAddressField(blank=True, default=None, null=True, protocol='IPv4'),
        ),
        migrations.AlterUniqueTogether(
            name='floatingip',
            unique_together=set([('tenant', 'address')]),
        ),
    ]
