# -*- coding: utf-8 -*-
# Generated by Django 1.11.16 on 2018-12-21 13:48
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('marketplace', '0048_add_request_type'),
    ]

    operations = [
        migrations.AddField(
            model_name='offering',
            name='billable',
            field=models.BooleanField(default=True, help_text='Purchase and usage is invoiced.'),
        ),
    ]
