# -*- coding: utf-8 -*-
# Generated by Django 1.11.7 on 2018-03-21 12:02
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('support', '0016_add_states'),
    ]

    operations = [
        migrations.AlterField(
            model_name='comment',
            name='backend_id',
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
        migrations.AlterField(
            model_name='issue',
            name='backend_id',
            field=models.CharField(blank=True, max_length=255, null=True, unique=True),
        ),
        migrations.AlterField(
            model_name='supportuser',
            name='backend_id',
            field=models.CharField(blank=True, max_length=255, null=True, unique=True),
        ),
        migrations.AlterUniqueTogether(
            name='comment',
            unique_together=set([('backend_id', 'issue')]),
        ),
    ]
