# -*- coding: utf-8 -*-
# Generated by Django 1.11.20 on 2019-05-22 12:02
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('openstack_tenant', '0045_volumeavailabilityzone_available'),
    ]

    operations = [
        migrations.AddField(
            model_name='instanceavailabilityzone',
            name='available',
            field=models.BooleanField(default=True),
        ),
    ]
