# -*- coding: utf-8 -*-
# Generated by Django 1.11.7 on 2018-07-03 17:02
from __future__ import unicode_literals

from django.db import migrations
import waldur_core.core.fields


class Migration(migrations.Migration):

    dependencies = [
        ('logging', '0001_squashed_0010'),
    ]

    operations = [
        migrations.AlterField(
            model_name='emailhook',
            name='event_groups',
            field=waldur_core.core.fields.JSONField(default=list, verbose_name='List of event groups'),
        ),
        migrations.AlterField(
            model_name='pushhook',
            name='event_groups',
            field=waldur_core.core.fields.JSONField(default=list, verbose_name='List of event groups'),
        ),
        migrations.AlterField(
            model_name='systemnotification',
            name='event_groups',
            field=waldur_core.core.fields.JSONField(default=list, verbose_name='List of event groups'),
        ),
        migrations.AlterField(
            model_name='webhook',
            name='event_groups',
            field=waldur_core.core.fields.JSONField(default=list, verbose_name='List of event groups'),
        ),
    ]
