<!DOCTYPE html>
<html lang="en">
<head>
<meta charset="UTF-8">
<meta http-equiv="X-UA-Compatible" content="IE=edge, chrome=1">
<meta name="viewport" content="width=device-width, initial-scale=1">

<title>Waldur MasterMind</title>
{% load staticfiles %}
<link rel="stylesheet" type="text/css" href="{% static "landing/css/style.css" %}">
</head>
<body>
    <h1 class="title">Waldur MasterMind</h1>

    <div class="well">
        <ul class="list-unstyled">
            <li><a class="waldur-link" href="http://docs.waldur.com/">Documentation</a></li>
            <li><a class="waldur-link" href="/api">API</a></li>
            <li><a class="waldur-link" href="/admin">Administration</a></li>
            <li><a class="waldur-link" href="/docs">API Documentation</a></li>
        </ul>
    </div>
</body>
</html>
