# -*- coding: utf-8 -*-
# Generated by Django 1.11.7 on 2018-07-03 17:43
from __future__ import unicode_literals

from django.db import migrations
import waldur_core.core.fields


class Migration(migrations.Migration):

    dependencies = [
        ('waldur_paypal', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='invoice',
            name='issuer_details',
            field=waldur_core.core.fields.JSONField(blank=True, default=dict, help_text='Stores data about invoice issuer'),
        ),
        migrations.AlterField(
            model_name='invoice',
            name='payment_details',
            field=waldur_core.core.fields.JSONField(blank=True, default=dict, help_text='Stores data about customer payment details'),
        ),
    ]
