<html>
<head lang="en">
    <meta charset="UTF-8">
    <title>Notifications from waldur_core</title>
</head>
<body>
<ul>
    {% for event in events %}
    <li>
        {{ event.message }}
        <div>{{ event.created|date:"M d H:m e"}}</div>
    </li>
    {% endfor %}
</ul>
</body>
</html>
