# -*- coding: utf-8 -*-
# Generated by Django 1.11.21 on 2019-08-21 08:45
from __future__ import unicode_literals

import django.contrib.postgres.fields.jsonb
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('marketplace', '0089_make_billing_period_non_nullable'),
    ]

    operations = [
        migrations.AddField(
            model_name='resource',
            name='current_usages',
            field=django.contrib.postgres.fields.jsonb.JSONField(blank=True, default=dict),
        ),
    ]
