# -*- coding: utf-8 -*-
# Generated by Django 1.11.7 on 2018-06-13 06:46
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('marketplace', '0003_attribute_section'),
    ]

    operations = [
        migrations.RenameField(
            model_name='offering',
            old_name='features',
            new_name='attributes',
        ),
    ]
