# -*- coding: utf-8 -*-
# Generated by Django 1.11.21 on 2019-07-03 12:11
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('waldur_vmware', '0009_customer_network'),
    ]

    operations = [
        migrations.AddField(
            model_name='virtualmachine',
            name='networks',
            field=models.ManyToManyField(blank=True, to='waldur_vmware.Network'),
        ),
    ]
