# -*- coding: utf-8 -*-
# Generated by Django 1.11.21 on 2019-07-22 11:58
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('structure', '0009_project_is_removed'),
        ('waldur_vmware', '0014_virtualmachine_folder'),
    ]

    operations = [
        migrations.AlterField(
            model_name='customerfolder',
            name='customer',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='structure.Customer'),
        ),
        migrations.AlterUniqueTogether(
            name='customerfolder',
            unique_together=set([('customer', 'folder')]),
        ),
    ]
