# -*- coding: utf-8 -*-
# Generated by Django 1.11.18 on 2019-02-11 10:56
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('invoices', '0027_migrate_items'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='offeringitem',
            name='invoice',
        ),
        migrations.RemoveField(
            model_name='offeringitem',
            name='offering',
        ),
        migrations.RemoveField(
            model_name='offeringitem',
            name='project',
        ),
        migrations.RemoveField(
            model_name='openstackitem',
            name='invoice',
        ),
        migrations.RemoveField(
            model_name='openstackitem',
            name='package',
        ),
        migrations.RemoveField(
            model_name='openstackitem',
            name='project',
        ),
        migrations.DeleteModel(
            name='OfferingItem',
        ),
        migrations.DeleteModel(
            name='OpenStackItem',
        ),
    ]
