# -*- coding: utf-8 -*-
# Generated by Django 1.11.14 on 2018-08-16 10:18
from __future__ import unicode_literals

import django.contrib.postgres.fields.jsonb
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import model_utils.fields
import waldur_core.core.fields
import waldur_core.core.validators


class Migration(migrations.Migration):

    dependencies = [
        ('support', '0021_protect_issue_user'),
    ]

    operations = [
        migrations.CreateModel(
            name='OfferingTemplate',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('name', models.CharField(max_length=150, unique=True, verbose_name='name')),
                ('uuid', waldur_core.core.fields.UUIDField()),
                ('config', django.contrib.postgres.fields.jsonb.JSONField()),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='offering',
            name='template',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='support.OfferingTemplate'),
        ),
        migrations.AlterField(
            model_name='offering',
            name='type',
            field=models.CharField(editable=False, max_length=255),
        ),
    ]
