# -*- coding: utf-8 -*-
# Generated by Django 1.11.21 on 2019-07-25 07:38
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('waldur_vmware', '0018_port_runtime_state'),
    ]

    operations = [
        migrations.AddField(
            model_name='port',
            name='service_project_link',
            field=models.ForeignKey(default=None, on_delete=django.db.models.deletion.PROTECT, related_name='+', to='waldur_vmware.VMwareServiceProjectLink'),
            preserve_default=False,
        ),
    ]
