# -*- coding: utf-8 -*-
# Generated by Django 1.11.7 on 2018-01-22 12:22
from __future__ import unicode_literals

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('structure', '0053_add_project_type'),
    ]

    operations = [
        migrations.AddField(
            model_name='customer',
            name='address',
            field=models.CharField(blank=True, max_length=300),
        ),
        migrations.AddField(
            model_name='customer',
            name='bank_account',
            field=models.CharField(blank=True, max_length=50),
        ),
        migrations.AddField(
            model_name='customer',
            name='bank_name',
            field=models.CharField(blank=True, max_length=150),
        ),
        migrations.AddField(
            model_name='customer',
            name='postal',
            field=models.CharField(blank=True, max_length=20),
        ),
        migrations.AddField(
            model_name='customer',
            name='type',
            field=models.CharField(blank=True, max_length=150),
        ),
        migrations.AddField(
            model_name='customer',
            name='accounting_start_date',
            field=models.DateTimeField(default=django.utils.timezone.now, verbose_name='Start date of accounting'),
        ),
        migrations.AddField(
            model_name='customer',
            name='default_tax_percent',
            field=models.DecimalField(decimal_places=2, default=0, max_digits=4,
                                      validators=[django.core.validators.MinValueValidator(0),
                                                  django.core.validators.MaxValueValidator(100)]),
        ),
    ]
