# -*- coding: utf-8 -*-
# Generated by Django 1.9.13 on 2017-05-05 15:39
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('structure', '0048_remove_balance'),
    ]

    operations = [
        migrations.AlterField(
            model_name='customer',
            name='abbreviation',
            field=models.CharField(blank=True, max_length=12),
        ),
    ]
