<html>
<head lang="en">
    <meta charset="UTF-8">
    <title>The issue you have created has a new comment</title>
</head>
<body>
<p>
    Hello!
</p>
<p>
    Please go to <a href="{{issue_url}}">{{ issue_url }}</a> to see it.
</p>
</body>
</html>
