# -*- coding: utf-8 -*-
# Generated by Django 1.11.16 on 2018-11-01 08:56
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('marketplace', '0042_order__file'),
    ]

    operations = [
        migrations.AlterField(
            model_name='offering',
            name='shared',
            field=models.BooleanField(default=True, help_text='Accessible to all customers.'),
        ),
    ]
